# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import RegisterDeviceView, UnRegisterDeviceView

urlpatterns = patterns(
    '',
    url(r'^register/$', RegisterDeviceView.as_view(),
        name='push-notifications-register'),
    url(r'^unregister/$', UnRegisterDeviceView.as_view(),
        name='push-notifications-unregister')
)



from django.core.exceptions import ImproperlyConfigured
import requests
from datetime import timedelta, datetime

from .base import BaseService

ZEROPUSH_REQUEST_URL = 'https://api.zeropush.com/notify'


class ZeroPushService(BaseService):

    def __init__(self, settings):
        self.auth_token = settings.get('AUTH_TOKEN')

        if not self.auth_token:
            raise ImproperlyConfigured('For ZeroPush to work you need an '
                                       'AUTH_TOKEN in the configuration')

    def send_push_notification(self, devices, message,
                               badge_number=None, sound=None,
                               payload=None, expiry=None):
        if len(devices):
            params = {
                "auth_token": self.auth_token,
                "device_tokens[]": [device.token for device in devices],
                "expiry": expiry,
                "sound": sound,
                "info": payload
            }

            for key in params.keys():
                if not params[key]:
                    del params[key]

            expiry_time = timedelta(days=30).seconds

            if isinstance(expiry, datetime):
                expiry_time = expiry.second
            elif isinstance(expiry, timedelta):
                expiry_time = expiry.seconds

            params['expiry'] = expiry_time

            response = requests.post(ZEROPUSH_REQUEST_URL, params)

            if response.ok:
                return True
        return False



# -*- coding: utf-8 -*-

# Third party
from rest_framework import serializers


class DeviceTokenSerializer(serializers.Serializer):
    token = serializers.CharField(label='Token', min_length=32, required=True)



from django.core.exceptions import ImproperlyConfigured


class InvalidPushNotificationError(ImproperlyConfigured):
    pass


class BaseService(object):

    def __init__(self, configuration):
        pass

    def send_push_notification(devices, message,
                               badge_number=None, sound=None,
                               payload=None, expiry=None):
        pass




from .models import PushDevice


def register_push_device(user, token, notice_types=None):
    return PushDevice.register_push_device(user, token, notice_types)


def send_push_notification(devices, message, **kwargs):
    return PushDevice.send_push_notification(devices, message, **kwargs)



from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.db.models.query import QuerySet

from .services import get_service


class PushDeviceMethods(object):
    def add_permissions(self, notice_types):
            self.model.batch_change_permissions(
                notice_types, self.all(), send=True)

    def remove_permissions(self, notice_types):
        self.model.batch_change_permissions(
            notice_types, self.all(), send=False)

    def register_push_device(self, user, token, notify_types=None):
        return self.model.register_push_device(user, token, notify_types)

    def send_push_notification(self, message, **kwargs):
        return self.model.send_push_notification(self.all(),
                                                 message, **kwargs)

    def with_permission(self, permission):
        return self.filter(notification_settings__name=permission)

    def with_permissions(self, permissions):
        return self.filter(notification_settings__name__in=permissions)

    def unregister_push_device(self, user, token):
        return self.model.unregister_push_device(user, token)


class PushDeviceManager(PushDeviceMethods, models.Manager):

    # This enables us to have add_permissions on the
    # RelatedManager
    use_for_related_fields = True

    def get_queryset(self):
        return self.model.QuerySet(self.model)


class PushDevice(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL,
                             verbose_name=_("Owner of this device"),
                             related_name="push_devices")
    token = models.CharField(_("Device token string"),
                             max_length=255, db_index=True)
    device_type = models.CharField(_("Type of device"),
                                   max_length=255, blank=True, null=True)

    objects = PushDeviceManager()

    def add_permissions(self, notice_types):
        """ Adds a permission to the push device """
        PushDevice.change_permissions(notice_types, self, send=True)

    def remove_permissions(self, notice_types):
        """ Removes a permission to the push device """
        PushDevice.change_permissions(notice_types, self, send=False)

    @classmethod
    def register_push_device(cls, user, token, notice_types=None):
        device, _ = cls.objects.get_or_create(token=token,
                                              user=user)
        if notice_types:
            cls.change_permissions(notice_types, device)
        return device

    @classmethod
    def unregister_push_device(cls, user, token):
        deleted = user.push_devices.filter(token=token).delete()
        return deleted > 0

    @classmethod
    def change_permissions(cls, notice_types, device, send=True):
        notice_types = (notice_types
                        if isinstance(notice_types, list) else [notice_types])

        for notice_type in notice_types:
            try:
                notification_setting = NotificationSetting.objects.get(
                    device=device, name=notice_type)
                notification_setting.send = send
                notification_setting.save()
            except NotificationSetting.DoesNotExist:
                notification_setting = NotificationSetting.objects.create(
                    device=device, name=notice_type, send=send)

    @classmethod
    def batch_change_permissions(cls, notice_types, devices, send=True):
        for device in devices:
            cls.change_permissions(notice_types, device, send)

    @classmethod
    def send_push_notification(cls, devices, message, **kwargs):
        return get_service().send_push_notification(devices, message, **kwargs)

    class QuerySet(PushDeviceMethods, QuerySet):
        pass

    def __unicode__(self):
        return u"Device %s" % self.token


class NotificationSetting(models.Model):
    device = models.ForeignKey(PushDevice, related_name='notification_settings')
    name = models.CharField(max_length=255, verbose_name=_("Notification name"))
    send = models.BooleanField(default=True)

    class Meta:
        unique_together = ('device', 'name')

    def __unicode__(self):
        return "{0} - {1}".format(self.device, self.type)

    @classmethod
    def can_send(cls, device, notice_type):
        """
        Check if the given device has the given `notice_type` enabled
        """
        try:
            notification_setting = cls.objects.get(device=device,
                                                   name=notice_type)
        except cls.DoesNotExist:
            notification_setting = cls.objects.create(device=device,
                                                      name=notice_type)

        return notification_setting.send



from django.conf import settings
from django.utils.module_loading import import_string
from django.core.exceptions import ImproperlyConfigured

# Local stuff
from .base import InvalidPushNotificationError

SERVICE_NAME_KEY = 'SERVICE'

if SERVICE_NAME_KEY not in settings.DJANGO_PUSH_NOTIFICATIONS:
    raise ImproperlyConfigured("You must define '%s' for "
                               "services to work" % SERVICE_NAME_KEY)


def get_service():
    """
    Autoloading service for getting the Service that is set up
    """
    return service


def _create_service(service_name):
    try:
        # Try to get the CACHES entry for the given backend name first
        service_cls = import_string(service_name)
    except ImportError as e:
        raise InvalidPushNotificationError(
            "Could not find service '%s': %s" % (service_name, e))

    return service_cls(settings.DJANGO_PUSH_NOTIFICATIONS)

service = _create_service(settings.DJANGO_PUSH_NOTIFICATIONS['SERVICE'])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='NotificationSetting',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=255, verbose_name='Notification name')),
                ('send', models.BooleanField(default=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PushDevice',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('token', models.CharField(max_length=255, verbose_name='Device token string', db_index=True)),
                ('device_type', models.CharField(max_length=255, null=True, verbose_name='Type of device', blank=True)),
                ('user', models.ForeignKey(related_name='push_devices', verbose_name='Owner of this device', to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='notificationsetting',
            name='device',
            field=models.ForeignKey(related_name='notification_settings', to='push_notifications.PushDevice'),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='notificationsetting',
            unique_together=set([('device', 'name')]),
        ),
    ]



# -*- coding: utf-8 -*-

# Third party
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework import status

# Local
from .models import PushDevice
from .serializers import DeviceTokenSerializer


class UnRegisterDeviceView(APIView):
    permission_classes = (IsAuthenticated,)

    def post(self, request, format=None):
        serializer = DeviceTokenSerializer(data=request.data)
        # check whether it's valid:
        if serializer.is_valid():
            PushDevice.unregister_push_device(
                request.user, serializer.validated_data['token'])
            return Response({'unregistered': True})

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class RegisterDeviceView(APIView):
    permission_classes = (IsAuthenticated,)

    def post(self, request, format=None):
        serializer = DeviceTokenSerializer(data=request.data)
        # check whether it's valid:
        if serializer.is_valid():
            PushDevice.register_push_device(
                request.user, serializer.validated_data['token'])
            return Response({'registered': True})

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import push_notifications


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = push_notifications.__version__

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()

readme = open('README.rst').read()

setup(
    name='dpnm',
    version=version,
    description="""A plug and play package to handle push devices and push notifications for services such as ZeroPush and Urban Airship""",
    long_description=readme,
    author='Paul Oostenrijk',
    author_email='paul@glemma.nl',
    url='https://github.com/Fueled/django-push-notifications',
    packages=get_packages('push_notifications'),
    include_package_data=True,
    install_requires=[
        'django>=1.5.1',
        'requests>=2.5.1',
        'djangorestframework>=3.1.1'
    ],
    license="BSD",
    zip_safe=False,
    keywords='pnm, django, push, notifications, manager',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
    ],
)
